"use strict";
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildResetValuesOnRecordUpdate = exports.buildSetStateFunction = exports.setFieldState = exports.buildNestedStateSet = exports.buildAccessChain = exports.buildUseStateExpression = exports.resetStateFunction = exports.getUseStateHooks = exports.getInitialValues = exports.getDefaultValueExpression = exports.setErrorState = exports.getSetNameIdentifier = exports.capitalizeFirstLetter = exports.setStateExpression = exports.resetValuesName = exports.getPropName = exports.getCurrentValueIdentifier = exports.getLinkedDataName = exports.getRecordsName = exports.getCurrentDisplayValueName = exports.getCurrentValueName = exports.getArrayChildRefName = void 0;
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const typescript_1 = require("typescript");
const helpers_1 = require("../../helpers");
const invalid_variable_helpers_1 = require("./invalid-variable-helpers");
const render_checkers_1 = require("./render-checkers");
// used just to sanitize nested array field names
// when rendering currentValue state and ref
const getVariableName = (input) => input.length > 1 ? input.join('').replace(/[^a-zA-Z0-9_$]/g, '') : input.join('');
const getArrayChildRefName = (fieldName) => {
    const paths = fieldName.split('.').map((path, i) => (i === 0 ? path : (0, exports.capitalizeFirstLetter)(path)));
    return `${getVariableName(paths)}Ref`;
};
exports.getArrayChildRefName = getArrayChildRefName;
// value of the child of an ArrayField
const getCurrentValueName = (fieldName) => {
    const paths = fieldName.split('.').map((path) => (0, exports.capitalizeFirstLetter)(path));
    return `current${getVariableName(paths)}Value`;
};
exports.getCurrentValueName = getCurrentValueName;
const getCurrentDisplayValueName = (fieldName) => `current${(0, exports.capitalizeFirstLetter)(fieldName)}DisplayValue`;
exports.getCurrentDisplayValueName = getCurrentDisplayValueName;
const getRecordsName = (modelName) => `${(0, helpers_1.lowerCaseFirst)(modelName)}Records`;
exports.getRecordsName = getRecordsName;
const getLinkedDataName = (modelName) => `linked${(0, exports.capitalizeFirstLetter)(modelName)}`;
exports.getLinkedDataName = getLinkedDataName;
const getCurrentValueIdentifier = (fieldName) => typescript_1.factory.createIdentifier((0, exports.getCurrentValueName)(fieldName));
exports.getCurrentValueIdentifier = getCurrentValueIdentifier;
// in update form, there will be conflict if field `id` is editable.
// so the prop `id` should be destructured as `idProp`
const getPropName = (propName) => `${propName}Prop`;
exports.getPropName = getPropName;
exports.resetValuesName = typescript_1.factory.createIdentifier('resetStateValues');
const setStateExpression = (fieldName, value) => {
    return typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression((0, exports.getSetNameIdentifier)(fieldName), undefined, [value]));
};
exports.setStateExpression = setStateExpression;
const capitalizeFirstLetter = (val) => {
    return val.charAt(0).toUpperCase() + val.slice(1);
};
exports.capitalizeFirstLetter = capitalizeFirstLetter;
const getSetNameIdentifier = (value) => {
    return typescript_1.factory.createIdentifier(`set${(0, exports.capitalizeFirstLetter)(value)}`);
};
exports.getSetNameIdentifier = getSetNameIdentifier;
/**
 * setErrors((errors) => ({ ...errors, [key]: value }));
 *
 * shorthand function to set error key/value
 * @param key ts expression to use as key ex. string literal or ts identifier
 * @param value ts expression to resolve to the value could be a prop access chain or identifier
 * @returns expression statement
 */
const setErrorState = (key, value) => {
    return typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('setErrors'), undefined, [
        typescript_1.factory.createArrowFunction(undefined, undefined, [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('errors'), undefined, undefined, undefined),
        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createParenthesizedExpression(typescript_1.factory.createObjectLiteralExpression([
            typescript_1.factory.createSpreadAssignment(typescript_1.factory.createIdentifier('errors')),
            typescript_1.factory.createPropertyAssignment(key, value),
        ], false))),
    ]));
};
exports.setErrorState = setErrorState;
/**
 * Note this is not for handling arrays.
 * If you need a default array value (e.g. []). Pass in an empty array at a higher level.
 */
const getDefaultValueExpression = (name, componentType, dataType, isArray, isDisplayValue) => {
    const componentTypeToDefaultValueMap = {
        Autocomplete: isDisplayValue ? typescript_1.factory.createStringLiteral('') : typescript_1.factory.createIdentifier('undefined'),
        ToggleButton: typescript_1.factory.createFalse(),
        SwitchField: typescript_1.factory.createFalse(),
        StepperField: typescript_1.factory.createNumericLiteral(0),
        SliderField: typescript_1.factory.createNumericLiteral(0),
        CheckboxField: typescript_1.factory.createFalse(),
    };
    if (isArray) {
        return typescript_1.factory.createArrayLiteralExpression([], false);
    }
    if (componentType in componentTypeToDefaultValueMap) {
        return componentTypeToDefaultValueMap[componentType];
    }
    return typescript_1.factory.createIdentifier('undefined');
};
exports.getDefaultValueExpression = getDefaultValueExpression;
/* ex. const initialValues = {
  name: undefined,
  isChecked: false
}
*/
const getInitialValues = (fieldConfigs) => {
    const stateNames = new Set();
    const propertyAssignments = Object.entries(fieldConfigs).reduce((acc, [name, { dataType, componentType, isArray }]) => {
        const isNested = name.includes('.');
        // we are only setting top-level keys
        const stateName = name.split('.')[0];
        let initialValue = (0, exports.getDefaultValueExpression)(name, componentType, dataType, isArray);
        if (isNested) {
            // if nested, just set up an empty object for the top-level key
            initialValue = typescript_1.factory.createObjectLiteralExpression();
        }
        if (!stateNames.has(stateName)) {
            acc.push(typescript_1.factory.createPropertyAssignment((0, codegen_ui_1.isValidVariableName)(stateName)
                ? typescript_1.factory.createIdentifier(stateName)
                : typescript_1.factory.createStringLiteral(stateName), initialValue));
            stateNames.add(stateName);
        }
        return acc;
    }, []);
    return typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('initialValues'), undefined, undefined, typescript_1.factory.createObjectLiteralExpression(propertyAssignments, true)),
    ], typescript_1.NodeFlags.Const));
};
exports.getInitialValues = getInitialValues;
/**
 * iterates field configs to create useState hooks for each field
 * @param fieldConfigs
 * @returns
 */
const getUseStateHooks = (fieldConfigs) => {
    const stateNames = new Set();
    return Object.entries(fieldConfigs).reduce((acc, [name, { sanitizedFieldName, dataType, isArray }]) => {
        const fieldName = name.split('.')[0];
        const renderedFieldName = sanitizedFieldName || fieldName;
        function determinePropertyName() {
            return (0, codegen_ui_1.isValidVariableName)(fieldName)
                ? typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('initialValues'), typescript_1.factory.createIdentifier(fieldName))
                : typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('initialValues'), typescript_1.factory.createStringLiteral(fieldName));
        }
        function renderCorrectUseStateValue() {
            if ((dataType === 'AWSJSON' || (0, codegen_ui_1.isNonModelDataType)(dataType)) && !isArray) {
                return typescript_1.factory.createConditionalExpression(determinePropertyName(), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('JSON'), typescript_1.factory.createIdentifier('stringify')), undefined, [determinePropertyName()]), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createIdentifier('undefined'));
            }
            return determinePropertyName();
        }
        if (!stateNames.has(renderedFieldName)) {
            acc.push((0, exports.buildUseStateExpression)(renderedFieldName, renderCorrectUseStateValue()));
            stateNames.add(renderedFieldName);
        }
        return acc;
    }, []);
};
exports.getUseStateHooks = getUseStateHooks;
/**
 * function used by the Clear/ Reset button
 * it's a reset type but we also need to clear the state of the input fields as well
 *
 * ex.
 * const resetStateValues = () => {
 *  setName(initialValues.name)
 *  setLastName(initialValues.lastName)
 *   ....
 * };
 */
const resetStateFunction = (fieldConfigs, recordName) => {
    const recordOrInitialValues = recordName ? 'cleanValues' : 'initialValues';
    const stateNames = new Set();
    const expressions = Object.entries(fieldConfigs).reduce((acc, [name, fieldConfig]) => {
        const { isArray, sanitizedFieldName, componentType, dataType } = fieldConfig;
        const stateName = name.split('.')[0];
        const renderedName = sanitizedFieldName || stateName;
        if (!stateNames.has(stateName)) {
            const accessExpression = (0, invalid_variable_helpers_1.getElementAccessExpression)(recordOrInitialValues, stateName);
            // Initial values should have the correct values and not need a modifier
            if ((dataType === 'AWSJSON' || (0, codegen_ui_1.isNonModelDataType)(dataType)) &&
                !isArray &&
                recordOrInitialValues !== 'initialValues') {
                const awsJSONAccessModifier = stringifyAWSJSONFieldValue(accessExpression);
                acc.push((0, exports.setStateExpression)(renderedName, awsJSONAccessModifier));
            }
            else {
                acc.push((0, exports.setStateExpression)(renderedName, isArray && recordOrInitialValues === 'cleanValues'
                    ? typescript_1.factory.createBinaryExpression(accessExpression, typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionQuestionToken), typescript_1.factory.createArrayLiteralExpression([], false))
                    : accessExpression));
            }
            if ((0, render_checkers_1.shouldWrapInArrayField)(fieldConfig)) {
                acc.push((0, exports.setStateExpression)((0, exports.getCurrentValueName)(renderedName), (0, exports.getDefaultValueExpression)(name, componentType, dataType)));
            }
            if ((0, render_checkers_1.isModelDataType)(fieldConfig)) {
                acc.push((0, exports.setStateExpression)((0, exports.getCurrentDisplayValueName)(renderedName), (0, exports.getDefaultValueExpression)(name, componentType, dataType, false, true)));
                stateNames.add(stateName);
            }
            stateNames.add(stateName);
        }
        return acc;
    }, []);
    const linkedDataPropertyAssignments = [];
    if (fieldConfigs) {
        Object.entries(fieldConfigs).forEach(([fieldName, fieldConfig]) => {
            var _a, _b, _c;
            if (((_a = fieldConfig.relationship) === null || _a === void 0 ? void 0 : _a.type) === 'HAS_MANY') {
                linkedDataPropertyAssignments.push(typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(fieldConfig.sanitizedFieldName || fieldName), typescript_1.factory.createIdentifier((0, exports.getLinkedDataName)(fieldName))));
            }
            else if (((_b = fieldConfig.relationship) === null || _b === void 0 ? void 0 : _b.type) === 'BELONGS_TO' || ((_c = fieldConfig.relationship) === null || _c === void 0 ? void 0 : _c.type) === 'HAS_ONE') {
                linkedDataPropertyAssignments.push(typescript_1.factory.createIdentifier(fieldConfig.sanitizedFieldName || fieldName));
            }
        });
    }
    // ex. const cleanValues = {...initialValues, ...bookRecord}
    if (recordName) {
        expressions.unshift(typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('cleanValues'), undefined, undefined, typescript_1.factory.createConditionalExpression(typescript_1.factory.createIdentifier(recordName), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createObjectLiteralExpression([
                typescript_1.factory.createSpreadAssignment(typescript_1.factory.createIdentifier('initialValues')),
                typescript_1.factory.createSpreadAssignment(typescript_1.factory.createIdentifier(recordName)),
                ...linkedDataPropertyAssignments,
            ], false), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createIdentifier('initialValues'))),
        ], typescript_1.NodeFlags.Const)));
    }
    // also reset the state of the errors
    expressions.push((0, exports.setStateExpression)('errors', typescript_1.factory.createObjectLiteralExpression()));
    return typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
        typescript_1.factory.createVariableDeclaration(exports.resetValuesName, undefined, undefined, typescript_1.factory.createArrowFunction(undefined, undefined, [], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock(expressions, true))),
    ], typescript_1.NodeFlags.Const));
};
exports.resetStateFunction = resetStateFunction;
/**
 * Datastore allows JSON strings and normal JSON so check for a string
 * before stringifying or else the string will return with escaped quotes
 *
 * Example output:
 * typeof cleanValues.metadata === 'string' ? cleanValues.metadata : JSON.stringify(cleanValues.metadata)
 */
const stringifyAWSJSONFieldValue = (value) => {
    return typescript_1.factory.createConditionalExpression(typescript_1.factory.createBinaryExpression(typescript_1.factory.createTypeOfExpression(value), typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsEqualsEqualsToken), typescript_1.factory.createStringLiteral('string')), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), value, typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('JSON'), typescript_1.factory.createIdentifier('stringify')), undefined, [value]));
};
/**
 * const [name, setName] = React.useState({default_expression});
 *
 * name is the value we are looking to set
 * defaultValue is is the value to set for the useState
 * @param name
 * @param defaultValue
 * @returns
 */
const buildUseStateExpression = (name, defaultValue) => {
    return typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createArrayBindingPattern([
            typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier(name), undefined),
            typescript_1.factory.createBindingElement(undefined, undefined, (0, exports.getSetNameIdentifier)(name), undefined),
        ]), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('React'), typescript_1.factory.createIdentifier('useState')), undefined, [defaultValue])),
    ], typescript_1.NodeFlags.Const));
};
exports.buildUseStateExpression = buildUseStateExpression;
/**
 * turns ['myNestedObject', 'value', 'nestedValue', 'leaf']
 *
 * into myNestedObject?.value?.nestedValue?.leaf
 *
 * @param values
 * @returns
 */
const buildAccessChain = (values, isOptional = true) => {
    if (values.length <= 0) {
        throw new Error('Need at least one value in the values array');
    }
    const optional = isOptional ? typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionDotToken) : undefined;
    if (values.length > 1) {
        const [parent, child, ...rest] = values;
        let propChain = typescript_1.factory.createPropertyAccessChain(typescript_1.factory.createIdentifier(parent), optional, typescript_1.factory.createIdentifier(child));
        if (!(0, codegen_ui_1.isValidVariableName)(child)) {
            propChain = typescript_1.factory.createElementAccessChain(typescript_1.factory.createIdentifier(parent), optional, typescript_1.factory.createStringLiteral(child));
        }
        if (rest.length) {
            rest.forEach((value) => {
                if ((0, codegen_ui_1.isValidVariableName)(value)) {
                    propChain = typescript_1.factory.createPropertyAccessChain(propChain, optional, typescript_1.factory.createIdentifier(value));
                }
                else {
                    propChain = typescript_1.factory.createElementAccessChain(propChain, optional, typescript_1.factory.createStringLiteral(value));
                }
            });
        }
        return propChain;
    }
    return typescript_1.factory.createIdentifier(values[0]);
};
exports.buildAccessChain = buildAccessChain;
const buildNestedStateSet = (keyPath, currentKeyPath, value, index = 1) => {
    if (keyPath.length <= 1) {
        throw new Error('keyPath needs a length larger than 1 to build nested state object');
    }
    const currentKey = keyPath[index];
    // the value of the index is what decides if we have reached the leaf property of the nested object
    if (keyPath.length - 1 === index) {
        return typescript_1.factory.createObjectLiteralExpression([
            typescript_1.factory.createSpreadAssignment((0, exports.buildAccessChain)(currentKeyPath)),
            typescript_1.factory.createPropertyAssignment((0, codegen_ui_1.isValidVariableName)(currentKey)
                ? typescript_1.factory.createIdentifier(currentKey)
                : typescript_1.factory.createComputedPropertyName(typescript_1.factory.createStringLiteral(currentKey)), value),
        ]);
    }
    const currentSpreadAssignment = (0, exports.buildAccessChain)(currentKeyPath);
    currentKeyPath.push(currentKey);
    return typescript_1.factory.createObjectLiteralExpression([
        typescript_1.factory.createSpreadAssignment(currentSpreadAssignment),
        typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(currentKey), (0, exports.buildNestedStateSet)(keyPath, currentKeyPath, value, index + 1)),
    ]);
};
exports.buildNestedStateSet = buildNestedStateSet;
// updating state
const setFieldState = (name, value) => {
    if (name.split('.').length > 1) {
        const keyPath = name.split('.');
        return typescript_1.factory.createCallExpression((0, exports.getSetNameIdentifier)(keyPath[0]), undefined, [
            (0, exports.buildNestedStateSet)(keyPath, [keyPath[0]], value),
        ]);
    }
    return typescript_1.factory.createCallExpression((0, exports.getSetNameIdentifier)(name), undefined, [value]);
};
exports.setFieldState = setFieldState;
const buildSetStateFunction = (fieldConfigs) => {
    const fieldSet = new Set();
    const expression = Object.keys(fieldConfigs).reduce((acc, field) => {
        const fieldName = field.split('.')[0];
        const renderedFieldName = fieldConfigs[field].sanitizedFieldName || fieldName;
        if (!fieldSet.has(renderedFieldName)) {
            acc.push(typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier(`set${(0, exports.capitalizeFirstLetter)(renderedFieldName)}`), undefined, [
                (0, codegen_ui_1.isValidVariableName)(fieldName)
                    ? typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('initialData'), typescript_1.factory.createIdentifier(fieldName))
                    : typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('initialData'), typescript_1.factory.createStringLiteral(fieldName)),
            ])));
            fieldSet.add(renderedFieldName);
        }
        return acc;
    }, []);
    return typescript_1.factory.createIfStatement(typescript_1.factory.createIdentifier('initialData'), typescript_1.factory.createBlock(expression, true));
};
exports.buildSetStateFunction = buildSetStateFunction;
// ex. React.useEffect(resetStateValues, [bookRecord])
const buildResetValuesOnRecordUpdate = (recordName, linkedDataNames) => {
    const linkedDataIdentifiers = [];
    linkedDataNames.forEach((linkedDataName) => {
        linkedDataIdentifiers.push(typescript_1.factory.createIdentifier(linkedDataName));
    });
    return typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('React'), typescript_1.factory.createIdentifier('useEffect')), undefined, [
        exports.resetValuesName,
        typescript_1.factory.createArrayLiteralExpression([typescript_1.factory.createIdentifier(recordName), ...linkedDataIdentifiers], false),
    ]));
};
exports.buildResetValuesOnRecordUpdate = buildResetValuesOnRecordUpdate;
//# sourceMappingURL=form-state.js.map